#include "mmas.h"
#include "eos/eos.h"
#include <gsl/gsl_errno.h>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_odeiv.h>
#include <gsl/gsl_interp.h>

#define EPSREL 1.0e-10

void mmas::solve_hse(usm& model, int mode) {
}
